/*
**   UltimateIRCd - Ultimate Internet Relay Chat Daemon, include/struct.h
**   Copyright (C) 1990 Jarkko Oikarinen and
**                      University of Oulu, Computing Center
**
**   Rewritten for UltimateIRCd by ShadowRealm Creations.
**   Copyright (C) 1997-2000 Infomedia Inc.
**
**   This program is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 1, or (at your option)
**   any later version.
**
**   This program is distributed in the hope that it will be useful,
**   but WITHOUT ANY WARRANTY; without even the implied warranty of
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**   GNU General Public License for more details.
**
**   You should have received a copy of the GNU General Public License
**   along with this program; if not, write to the Free Software
**   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef	__struct_include__
#define __struct_include__

#include "config.h"
#include "common.h"
#include "sys.h"

#include <stdio.h>
#include <sys/types.h>
#ifndef _WIN32
#include <netinet/in.h>
#include <netdb.h>
#endif
#ifdef STDDEFH
# include <stddef.h>
#endif

#ifdef USE_SYSLOG
# include <syslog.h>
# ifdef SYSSYSLOGH
#  include <sys/syslog.h>
# endif
#endif
#ifdef	pyr
#include <sys/time.h>
#endif

typedef struct  t_crline aCRline;

typedef struct  s_gline gline_t;
typedef struct  t_gline aGline;
typedef struct  SqlineItem aSqlineItem;
typedef	struct	ConfItem aConfItem;
typedef struct	Notify	aNotify;
typedef	struct 	Client	aClient;
typedef	struct	Channel	aChannel;
typedef	struct	User	anUser;
typedef	struct	Server	aServer;
typedef	struct	SLink	Link;
typedef struct	SBan	Ban;
typedef	struct	SMode	Mode;
typedef struct	ListOptions	LOpts;

typedef struct  FloodOpt  aFloodOpt;
typedef struct  CloneItem aClone;

#ifdef NEED_U_INT32_T
typedef unsigned int  u_int32_t; /* XXX Hope this works! */
#endif

#ifndef VMSP
#include "class.h"
#include "dbuf.h"	/* THIS REALLY SHOULDN'T BE HERE!!! --msa */
#endif


/*
**
** Original HOSTLEN value = 63.
** Increased due to the Virtualhost taking up
** some of the max lenght on hostnames bordering to the max.
** - HAMLET
**
*/

#define	HOSTLEN		75	/* Length of hostname.  Updated to         */
				/* comply with RFC1123                     */

#define	USERLEN		10
#define	REALLEN	 	50
#define	REALLEN2	80
#define	TOPICLEN	307
/* DAL MADE ME PUT THIS IN THE FIEND:  --Russell
 * This number will be expanded to 200 in the near future
 */
#define	CHANNELLEN	32
#define	PASSWDLEN 	32	/* orig. 20, changed to 32 for nickpasswords */
#define	KEYLEN		23
#define LINKLEN		32
#define	BUFSIZE		512		/* WARNING: *DONT* CHANGE THIS!!!! */
#define	MAXRECIPIENTS 	20
#define	MAXKILLS	20
#define	MAXBANS		60
#define	MAXBANLENGTH	1024
#define	MAXSILES	5
#define	MAXSILELENGTH	128
/*
 * Watch it - Don't change this unless you also change the ERR_TOOMANYWATCH
 * and PROTOCOL_SUPPORTED settings.
 */
#define MAXWATCH	128

#define	USERHOST_REPLYLEN	(32+HOSTLEN+USERLEN+5)



/* NOTE: this must be down here so the stuff from struct.h IT uses works */
#include "whowas.h"


/*
** 'offsetof' is defined in ANSI-C. The following definition
** is not absolutely portable (I have been told), but so far
** it has worked on all machines I have needed it. The type
** should be size_t but...  --msa
*/
#ifndef offsetof
#define	offsetof(t,m) (int)((&((t *)0L)->m))
#endif

#define	elementsof(x) (sizeof(x)/sizeof(x[0]))

/*
** flags for bootup options (command line flags)
*/
#define	BOOT_CONSOLE	1
#define	BOOT_QUICK	2
#define	BOOT_DEBUG	4
#define	BOOT_INETD	8
#define	BOOT_TTY	16
#define	BOOT_OPER	32
#define	BOOT_AUTODIE	64

#define STAT_PING       -7	/* for UPING -GZ */
#define	STAT_LOG	-6	/* logfile for -x */
#define	STAT_MASTER	-5	/* Local ircd master before identification */
#define	STAT_CONNECTING	-4
#define	STAT_HANDSHAKE	-3
#define	STAT_ME		-2
#define	STAT_UNKNOWN	-1
#define	STAT_SERVER	0
#define	STAT_CLIENT	1
#define	STAT_SERVICE	2	/* Services not implemented yet */

/*
 * status macros.
 */
#define	IsRegisteredUser(x)	((x)->status == STAT_CLIENT)
#define	IsRegistered(x)		((x)->status >= STAT_SERVER)
#define	IsConnecting(x)		((x)->status == STAT_CONNECTING)
#define	IsHandshake(x)		((x)->status == STAT_HANDSHAKE)
#define	IsMe(x)			((x)->status == STAT_ME)
#define	IsUnknown(x)		((x)->status == STAT_UNKNOWN || \
				 (x)->status == STAT_MASTER)
#define	IsServer(x)		((x)->status == STAT_SERVER)
#define	IsClient(x)		((x)->status == STAT_CLIENT)
#define	IsLog(x)		((x)->status == STAT_LOG)
#define IsService(x)		0

#define	SetMaster(x)		((x)->status = STAT_MASTER)
#define	SetConnecting(x)	((x)->status = STAT_CONNECTING)
#define	SetHandshake(x)		((x)->status = STAT_HANDSHAKE)
#define	SetMe(x)		((x)->status = STAT_ME)
#define	SetUnknown(x)		((x)->status = STAT_UNKNOWN)
#define	SetServer(x)		((x)->status = STAT_SERVER)
#define	SetClient(x)		((x)->status = STAT_CLIENT)
#define	SetLog(x)		((x)->status = STAT_LOG)
#define	SetService(x)		((x)->status = STAT_SERVICE)

#define	FLAGS_PINGSENT   	0x0001	/* Unreplied ping sent */
#define	FLAGS_DEADSOCKET 	0x0002	/* Local socket is dead--Exiting soon */
#define	FLAGS_KILLED     	0x0004	/* Prevents "QUIT" from being sent for this */
#define	FLAGS_BLOCKED    	0x0008	/* socket is in a blocked condition */
#define	FLAGS_UNIX	 	0x0010	/* socket is in the unix domain, not inet */
#define	FLAGS_CLOSING    	0x0020	/* set when closing to suppress errors */
#define	FLAGS_LISTEN     	0x0040 /* used to mark clients which we listen() on */
#define	FLAGS_CHKACCESS  	0x0080 /* ok to check clients access if set */
#define	FLAGS_DOINGDNS	 	0x0100 /* client is waiting for a DNS response */
#define	FLAGS_AUTH	 	0x0200 /* client is waiting on rfc931 response */
#define	FLAGS_WRAUTH	 	0x0400	/* set if we havent writen to ident server */
#define	FLAGS_LOCAL	 	0x0800 /* set for local clients */
#define	FLAGS_GOTID	 	0x1000	/* successful ident lookup achieved */
#define	FLAGS_DOID	 	0x2000	/* I-lines say must use ident return */
#define	FLAGS_NONL	 	0x4000 /* No \n in buffer */
#define FLAGS_TS8	 	0x8000 /* Why do you want to know? */
#define FLAGS_ULINE		0x10000 /* User/server is considered U-lined */
#define FLAGS_SQUIT		0x20000 /* Server has been /squit by an oper */
#define FLAGS_PROTOCTL		0x40000 /* Received a PROTOCTL message */
#define FLAGS_MAP		0x80000 /* Map command */
#define FLAGS_SOCKS    		0x100000 /* client is waiting on a rfc1928 response */
#define FLAGS_WRSOCKS  		0x200000 /* set if we havent written to socks server */
#define	FLAGS_GOTSOCKS 		0x400000 /* client has an open socks server */
#define FLAGS_SNETINFO 		0x800000
#define FLAGS_VCTRL    		0x1000000
#define FLAGS_PING    		0x2000000 /* UPING SENT */
#define FLAGS_ASKEDPING 	0x4000000 /* UPONG GOT*/


/* Dec 26th, 1997 - added flags2 when I ran out of room in flags -DuffJ 

/* Dec 26th, 1997 - having a go at
 * splitting flags into flags and umodes
 * -DuffJ
 */

#define	UMODE_INVISIBLE  	0x0001 /* makes user invisible */
#define	UMODE_OPER       	0x0002	/* Operator */
#define	UMODE_WALLOP     	0x0004 /* send wallops to them */
#define UMODE_FAILOP	 	0x0008 /* Shows some global messages */
#define UMODE_HELPOP	 	0x0010 /* Help system operator */
#define UMODE_REGNICK	 	0x0020 /* Nick set by services as registered */
#define UMODE_SERVICESOPER	0x0040 /* Services Oper */
#define UMODE_ADMIN	 	0x0080 /* Admin */

#define	UMODE_SERVNOTICE 	0x0100 /* server notices such as kill */
#define	UMODE_LOCOP      	0x0200 /* Local operator -- SRB */
#define UMODE_KILLS	 	0x0400 /* Show server-kills... */
#define UMODE_CLIENT	 	0x0800 /* Show client information */
#define UMODE_FLOOD	 	0x1000 /* Receive flood warnings */
#define UMODE_CHATOP	 	0x2000 /* can receive chatops */
#define UMODE_SERVICES   	0x4000 /* services */
#define UMODE_HIDE	 	0x8000 /* Hide from Nukes */
#define UMODE_NETADMIN  	0x10000 /* Network Admin */
#define	UMODE_SUPER		0x20000 /* Oper Is Protected from Kick's and Kill's */
#define UMODE_RBOT      	0x40000 /* Marks the client as a Registered Bot */
#define UMODE_SBOT      	0x80000 /* Marks the client as a Server Bot */
#define UMODE_NGLOBAL  		0x100000 /* See Network Globals */
#define UMODE_WHOIS    		0x200000 /* Lets Opers see when people do a /WhoIs on them */
#define UMODE_NETINFO  		0x400000 /* Server link, Delink Notces etc. */
#define UMODE_MAGICK   		0x800000 /* Allows Opers To See +s and +p Channels */
#define UMODE_IRCADMIN 		0x1000000 /* Marks the client as an IRC Administrator */
#define UMODE_SERVICESADMIN	0x2000000 /* Marks the client as a Services Administrator */
#define UMODE_WATCHER		0x4000000 /* Recive Monitor Globals */
#define UMODE_NETMON		0x8000000 /* Marks the client as an Network Monitor */
#define UMODE_SERVADMIN		0x10000000 /* Marks the client as a Server Admin */
#define UMODE_TECHADMIN		0x20000000 /* Marks the client as a Technical Admin */



#define	SEND_UMODES	(UMODE_INVISIBLE|UMODE_OPER|UMODE_LOCOP|UMODE_WALLOP|UMODE_FAILOP|UMODE_HELPOP|UMODE_REGNICK|UMODE_SERVICESOPER|UMODE_NETADMIN|UMODE_SUPER|UMODE_ADMIN|UMODE_SERVICES|UMODE_HIDE|UMODE_RBOT|UMODE_SBOT|UMODE_NGLOBAL|UMODE_WHOIS|UMODE_NETINFO|UMODE_MAGICK|UMODE_IRCADMIN|UMODE_SERVICESADMIN|UMODE_WATCHER|UMODE_NETMON|UMODE_SERVADMIN|UMODE_TECHADMIN)
#define	ALL_UMODES	(SEND_UMODES|UMODE_SERVNOTICE|UMODE_KILLS|UMODE_CLIENT|UMODE_FLOOD|UMODE_CHATOP|UMODE_SERVICES|UMODE_RBOT|UMODE_SBOT|UMODE_NGLOBAL|UMODE_WHOIS|UMODE_NETINFO|UMODE_MAGICK|UMODE_IRCADMIN|UMODE_SERVICESADMIN|UMODE_WATCHER|UMODE_NETMON|UMODE_SERVADMIN|UMODE_TECHADMIN)
#define	FLAGS_ID	(FLAGS_DOID|FLAGS_GOTID)

#define PROTO_NOQUIT	0x1	/* Negotiated NOQUIT protocol */
#define PROTO_TOKEN	0x2	/* Negotiated TOKEN protocol */

/*
 * flags macros.
 */
#define IsKillsF(x)		((x)->umodes & UMODE_KILLS)
#define IsClientF(x)		((x)->umodes & UMODE_CLIENT)
#define IsFloodF(x)		((x)->umodes & UMODE_FLOOD)
#define IsRBot(x)		((x)->umodes & UMODE_RBOT)
#define IsSBot(x)		((x)->umodes & UMODE_SBOT)
#define IsWhois(x)		((x)->umodes & UMODE_WHOIS)
#define SendNetInfo(x)		((x)->umodes & UMODE_NETINFO)
#define IsMagick(x)		((x)->umodes & UMODE_MAGICK)
#define IsHelpOp(x)		((x)->umodes & UMODE_HELPOP)
#define IsAdmin(x)		((x)->umodes & UMODE_ADMIN)
#define IsNetAdmin(x)		((x)->umodes & UMODE_NETADMIN)
#define IsSuper(x)		((x)->umodes & UMODE_SUPER)
#define IsServicesOper(x)	((x)->umodes & UMODE_SERVICESOPER)
#define SendNetGlobal(x)	((x)->umodes & UMODE_NGLOBAL)
#define SendFailops(x)		((x)->umodes & UMODE_FAILOP)
#define SendChatops(x)		((x)->umodes & UMODE_CHATOP)
#define IsGlobops(x)		((x)->umodes & UMODE_FAILOP)
#define	IsOper(x)		((x)->umodes & UMODE_OPER)
#define	IsLocOp(x)		((x)->umodes & UMODE_LOCOP)
#define	IsInvisible(x)		((x)->umodes & UMODE_INVISIBLE)
#define IsServices(x)		((x)->umodes & UMODE_SERVICES)
#define	IsAnOper(x)		((x)->umodes & (UMODE_OPER|UMODE_LOCOP))
#define IsARegNick(x)		((x)->umodes & (UMODE_REGNICK))
#define IsRegNick(x)		((x)->umodes & UMODE_REGNICK)
#define	IsPerson(x)		((x)->user && IsClient(x))
#define	IsPrivileged(x)		(IsAnOper(x) || IsServer(x))
#define	SendWallops(x)		((x)->umodes & UMODE_WALLOP)
#define	SendServNotice(x)	((x)->umodes & UMODE_SERVNOTICE)
#define	IsUnixSocket(x)		((x)->flags & FLAGS_UNIX)
#define	IsListening(x)		((x)->flags & FLAGS_LISTEN)
#define	DoAccess(x)		((x)->flags & FLAGS_CHKACCESS)
#define	IsLocal(x)		((x)->flags & FLAGS_LOCAL)
#define	IsDead(x)		((x)->flags & FLAGS_DEADSOCKET)
#define GotProtoctl(x)		((x)->flags & FLAGS_PROTOCTL)
#define IsBlocked(x)		((x)->flags & FLAGS_BLOCKED)
#define IsHidden(x)             ((x)->umodes & UMODE_HIDE)
#define IsIRCAdmin(x)		((x)->umodes & UMODE_IRCADMIN)
#define IsServicesAdmin(x)	((x)->umodes & UMODE_SERVICESADMIN)
#define IsWatcher(x)		((x)->umodes & UMODE_WATCHER)
#define IsNetMon(x)		((x)->umodes & UMODE_NETMON)
#define GotSNetInfo(x) 		((x)->flags & FLAGS_SNETINFO)
#define SetSNetInfo(x)		((x)->flags |= FLAGS_SNETINFO)
#define GotVCtrl(x) 		((x)->flags & FLAGS_VCTRL)
#define SetVCtrl(x)		((x)->flags |= FLAGS_VCTRL)
#define IsServAdmin(x)		((x)->umodes & UMODE_SERVADMIN)
#define IsTechAdmin(x)		((x)->umodes & UMODE_TECHADMIN)

/* is some kind of admin */
#define IsSkoAdmin(x) 		(IsAdmin(x) || IsNetAdmin(x) || IsIRCAdmin(x) || IsServAdmin(x) || IsServicesAdmin(x) || IsTechAdmin(x))
#define IsSkoBot(x) 		(IsRBot(x) || IsSBot(x))

#ifdef NOSPOOF
#define	IsNotSpoof(x)		((x)->nospoof == 0)
#else
#define IsNotSpoof(x)           (1)
#endif

#define SetKillsF(x)		((x)->umodes |= UMODE_KILLS)
#define SetClientF(x)		((x)->umodes |= UMODE_CLIENT)
#define SetFloodF(x)		((x)->umodes |= UMODE_FLOOD)
#define SetHelpOp(x)		((x)->umodes |= UMODE_HELPOP)
#define	SetOper(x)		((x)->umodes |= UMODE_OPER)
#define	SetLocOp(x)    		((x)->umodes |= UMODE_LOCOP)
#define SetServicesOper(x)	((x)->umodes |= UMODE_SERVICESOPER)
#define SetAdmin(x)		((x)->umodes |= UMODE_ADMIN)
#define SetNetAdmin(x)		((x)->umodes |= UMODE_NETADMIN)
#define	SetInvisible(x)		((x)->umodes |= UMODE_INVISIBLE)
#define SetRBot(x)		((x)->umodes |= UMODE_RBOT)
#define SetSBot(x)		((x)->umodes |= UMODE_SBOT)
#define SetWhois(x)		((x)->umodes |= UMODE_WHOIS)
#define SetMagick(x)		((x)->umodes |= UMODE_MAGICK)
#define SetSuper(x)		((x)->umodes |= UMODE_SUPER)
#define SetNetInfo(x)		((x)->umodes |= UMODE_NETINFO)
#define	SetWallops(x)  		((x)->umodes |= UMODE_WALLOP)
#define SetGlobops(x)		((x)->umodes |= UMODE_FAILOP)
#define SetNetGlobal(x)		((x)->umodes |= UMODE_NGLOBAL)
#define	SetUnixSock(x)		((x)->flags |= FLAGS_UNIX)
#define	SetDNS(x)		((x)->flags |= FLAGS_DOINGDNS)
#define	DoingDNS(x)		((x)->flags & FLAGS_DOINGDNS)
#define	SetAccess(x)		((x)->flags |= FLAGS_CHKACCESS)
#define SetBlocked(x)		((x)->flags |= FLAGS_BLOCKED)
#define	DoingAuth(x)		((x)->flags & FLAGS_AUTH)
#define	DoingSocks(x)		((x)->flags & FLAGS_SOCKS)
#define	NoNewLine(x)		((x)->flags & FLAGS_NONL)
#define SetRegNick(x)		((x)->umodes & UMODE_REGNICK)
#define SetHidden(x)            ((x)->umodes |= UMODE_HIDE)
#define SetIRCAdmin(x)		((x)->umodes |= UMODE_IRCADMIN)
#define SetServicesAdmin(x)	((x)->umodes |= UMODE_SERVICESADMIN)
#define SetWatcher(x)		((x)->umodes |= UMODE_WATCHER)
#define SetNetMon(x)		((x)->umodes |= UMODE_NETMON)
#define SetServAdmin(x)		((x)->umodes |= UMODE_SERVADMIN)
#define SetTechAdmin(x)		((x)->umodes |= UMODE_TECHADMIN)
#define SetServNotice(x)	((x)->umodes |= UMODE_SERVNOTICE)
#define SetChatop(x)		((x)->umodes |= UMODE_CHATOP)

#define ClearAdmin(x)		((x)->umodes &= ~UMODE_ADMIN)
#define ClearNetAdmin(x)	((x)->umodes &= ~UMODE_NETADMIN)
#define ClearSuper(x)		((x)->umodes &= ~UMODE_SUPER)
#define ClearServicesOper(x)	((x)->umodes &= ~UMODE_SERVICESOPER)
#define ClearKillsF(x)		((x)->umodes &= ~UMODE_KILLS)
#define ClearClientF(x)		((x)->umodes &= ~UMODE_CLIENT)
#define ClearFloodF(x)		((x)->umodes &= ~UMODE_FLOOD)
#define ClearRBot(x)		((x)->umodes &= ~UMODE_RBOT)
#define ClearSBot(x)		((x)->umodes &= ~UMODE_SBOT)
#define ClearWhois(x)		((x)->umodes &= ~UMODE_WHOIS)
#define ClearNetInfo(x)		((x)->umodes &= ~UMODE_NETINFO)
#define ClearMagick(x)		((x)->umodes &= ~UMODE_MAGICK)
#define ClearHelpOp(x)		((x)->umodes &= ~UMODE_HELPOP)
#define ClearNetGlobal(x)	((x)->umodes &= ~UMODE_NGLOBAL)
#define ClearFailops(x)		((x)->umodes &= ~UMODE_FAILOP)
#define ClearChatops(x)		((x)->umodes &= ~UMODE_CHATOP)
#define	ClearOper(x)		((x)->umodes &= ~UMODE_OPER)
#define	ClearInvisible(x)	((x)->umodes &= ~UMODE_INVISIBLE)
#define ClearServices(x)	((x)->umodes &= ~UMODE_SERVICES)
#define	ClearWallops(x)		((x)->umodes &= ~UMODE_WALLOP)
#define ClearGlobops(x)		((x)->umodes &= ~UMODE_FAILOP)
#define	ClearDNS(x)		((x)->flags &= ~FLAGS_DOINGDNS)
#define	ClearAuth(x)		((x)->flags &= ~FLAGS_AUTH)
#define	ClearSocks(x)		((x)->flags &= ~FLAGS_SOCKS)
#define	ClearAccess(x)		((x)->flags &= ~FLAGS_CHKACCESS)
#define ClearBlocked(x)		((x)->flags &= ~FLAGS_BLOCKED)
#define ClearHidden(x)          ((x)->umodes &= ~UMODE_HIDE)
#define ClearIRCAdmin(x)	((x)->umodes &= ~UMODE_IRCADMIN)
#define ClearServicesAdmin(x)	((x)->umodes &= ~UMODE_SERVICESADMIN)
#define ClearWatcher(x)		((x)->umodes &= ~UMODE_WATCHER)
#define ClearNetMon(x)		((x)->umodes &= ~UMODE_NETMON)
#define ClearServAdmin(x)	((x)->umodes &= ~UMODE_SERVADMIN)
#define ClearTechAdmin(x)	((x)->umodes &= ~UMODE_TECHADMIN)


/*
**
** December 18th 1999
** Ran out of space, added second usermode container.
** Coded by QUINN. - HAMLET
**
*/

#define UMODE_GCLIENT		0x1 /* Global User Connect Notices */
#define UMODE_SERVICESROOT	0x2 /* Services Root Administrator */
#define UMODE_DEAF		0x4 /* Client Is Deaf on #Channels */


#define	SEND_UMODES2	(UMODE_GCLIENT|UMODE_SERVICESROOT|UMODE_DEAF)
#define	ALL_UMODES2	(SEND_UMODES2)


/*
**
** flags macros.
**
*/


#define IsGClient(x)		((x)->umodes2 & UMODE_GCLIENT)
#define IsServicesRoot(x)	((x)->umodes2 & UMODE_SERVICESROOT)
#define IsDeaf(x)		((x)->umodes2 & UMODE_DEAF)


#define SetGClient(x)		((x)->umodes2 |= UMODE_GCLIENT)
#define SetServicesRoot(x)	((x)->umodes2 |= UMODE_SERVICESROOT)
#define SetDeaf(x)		((x)->umodes2 |= UMODE_DEAF)


#define ClearGClient(x)		((x)->umodes2 &= ~UMODE_GCLIENT)
#define ClearServicesRoot(x)	((x)->umodes2 &= ~UMODE_SERVICESROOT)
#define ClearDeaf(x)		((x)->umodes2 &= ~UMODE_DEAF)


/*
 * ProtoCtl options
 */
#define DontSendQuit(x)		((x)->proto & PROTO_NOQUIT)
#define IsToken(x)		((x)->proto & PROTO_TOKEN)

#define SetNoQuit(x)		((x)->proto |= PROTO_NOQUIT)
#define SetToken(x)		((x)->proto |= PROTO_TOKEN)

#define ClearNoQuit(x)		((x)->proto &= ~PROTO_NOQUIT)
#define ClearToken(x)		((x)->proto &= ~PROTO_TOKEN)


/*
 * defined operator access levels
 */
#define OFLAG_REHASH		0x00000001  /* Oper can /rehash server */
#define OFLAG_DIE		0x00000002  /* Oper can /die the server */
#define OFLAG_RESTART		0x00000004  /* Oper can /restart the server */
#define OFLAG_HELPOP		0x00000010  /* Oper can send /HelpOps */
#define OFLAG_GLOBOP		0x00000020  /* Oper can send /GlobOps */
#define OFLAG_WALLOP		0x00000040  /* Oper can send /WallOps */
#define OFLAG_LOCOP		0x00000080  /* Oper can send /LocOps */
#define OFLAG_LROUTE		0x00000100  /* Oper can do local routing */
#define OFLAG_GROUTE		0x00000200  /* Oper can do global routing */
#define OFLAG_LKILL		0x00000400  /* Oper can do local kills */
#define OFLAG_GKILL		0x00000800  /* Oper can do global kills */
#define OFLAG_KLINE		0x00001000  /* Oper can /kline users */
#define OFLAG_UNKLINE		0x00002000  /* Oper can /unkline users */
#define OFLAG_LNOTICE		0x00004000  /* Oper can send local serv notices */
#define OFLAG_GNOTICE		0x00008000  /* Oper can send global notices */
#define OFLAG_ADMIN		0x00010000  /* Admin */
#define OFLAG_UMODEC		0x00020000  /* Oper can set umode +c */
#define OFLAG_UMODEF		0x00040000  /* Oper can set umode +f */
#define OFLAG_ZLINE		0x00080000  /* Oper can use /zline and /unzline */
#define OFLAG_SERVICESOPER    	0x00100000  /* Oper is a services oper */
#define OFLAG_EDCONF		0x00200000  /* Oper can use /addop etc */
#define OFLAG_IRCADMIN  	0x00400000  /* IRC Admin */
#define OFLAG_SERVICESADMIN 	0x00800000  /* Services Admin */
#define OFLAG_NETADMIN		0x01000000  /* Network Admin */
#define OFLAG_SERVADMIN		0x02000000  /* Serv Admin */
#define OFLAG_TECHADMIN		0x04000000  /* Tech Admin */
#define OFLAG_SERVICESROOT	0x08000000  /* Services Root Admin */
#define OFLAG_LOCAL		0x10000000  /* Marks The Oper As a Local Oper Only */
#define OFLAG_PROTECTED		0x20000000  /* Allows the Oper to set +p Protected mode */
#define OFLAG_GLOBAL		0x40000000  /* Marks The Oper As A GLOBAL Operator */

//#define OFLAG_GLOBAL	(OFLAG_REHASH|OFLAG_HELPOP|OFLAG_GLOBOP|OFLAG_WALLOP|OFLAG_LOCOP|OFLAG_LROUTE|OFLAG_LKILL|OFLAG_KLINE|OFLAG_UNKLINE|OFLAG_LNOTICE|OFLAG_UMODEC|OFLAG_UMODEF|OFLAG_GROUTE|OFLAG_GKILL|OFLAG_GNOTICE)
#define OFLAG_ISGLOBAL	(OFLAG_GROUTE|OFLAG_GKILL|OFLAG_GNOTICE)

#define OFLAG_FASTLOCOP		(OFLAG_LOCAL|OFLAG_HELPOP|OFLAG_WALLOP|OFLAG_LOCOP|OFLAG_LKILL|OFLAG_LNOTICE)
#define OFLAG_FASTOPER		(OFLAG_GLOBAL|OFLAG_HELPOP|OFLAG_GLOBOP|OFLAG_WALLOP|OFLAG_LOCOP|OFLAG_LKILL|OFLAG_GKILL|OFLAG_UMODEC|OFLAG_UMODEF)
#define OFLAG_FASTADMIN		(OFLAG_FASTOPER|OFLAG_ADMIN|OFLAG_KLINE|OFLAG_UNKLINE|OFLAG_ZLINE|OFLAG_LNOTICE|OFLAG_GNOTICE|OFLAG_LROUTE|OFLAG_GROUTE)
#define OFLAG_FASTSERVERADMIN	(OFLAG_FASTADMIN|OFLAG_REHASH|OFLAG_RESTART|OFLAG_DIE|OFLAG_EDCONF|OFLAG_SERVADMIN)
#define OFLAG_FASTTECHADMIN	(OFLAG_FASTSERVERADMIN|OFLAG_TECHADMIN)
#define OFLAG_FASTNETADMIN 	(OFLAG_FASTSERVERADMIN|OFLAG_NETADMIN|OFLAG_TECHADMIN)
#define OFLAG_FASTIRCADMIN 	(OFLAG_FASTADMIN|OFLAG_IRCADMIN)






#define OpCanEDConf(x)		((x)->oflag & OFLAG_EDCONF)
#define OPCanZline(x)   	((x)->oflag & OFLAG_ZLINE)
#define OPCanRehash(x)		((x)->oflag & OFLAG_REHASH)
#define OPCanDie(x)		((x)->oflag & OFLAG_DIE)
#define OPCanRestart(x)		((x)->oflag & OFLAG_RESTART)
#define OPCanHelpOp(x)		((x)->oflag & OFLAG_HELPOP)
#define OPCanGlobOps(x)		((x)->oflag & OFLAG_GLOBOP)
#define OPCanWallOps(x)		((x)->oflag & OFLAG_WALLOP)
#define OPCanLocOps(x)		((x)->oflag & OFLAG_LOCOP)
#define OPCanLRoute(x)		((x)->oflag & OFLAG_LROUTE)
#define OPCanGRoute(x)		((x)->oflag & OFLAG_GROUTE)
#define OPCanLKill(x)		((x)->oflag & OFLAG_LKILL)
#define OPCanGKill(x)		((x)->oflag & OFLAG_GKILL)
#define OPCanKline(x)		((x)->oflag & OFLAG_KLINE)
#define OPCanUnKline(x)		((x)->oflag & OFLAG_UNKLINE)
#define OPCanLNotice(x)		((x)->oflag & OFLAG_LNOTICE)
#define OPCanGNotice(x)		((x)->oflag & OFLAG_GNOTICE)
#define OPIsAdmin(x)		((x)->oflag & OFLAG_ADMIN)
#define OPIsServicesOper(x)	((x)->oflag & OFLAG_SERVICESOPER)
#define OPCanUModeC(x)		((x)->oflag & OFLAG_UMODEC)
#define OPCanUModeF(x)		((x)->oflag & OFLAG_UMODEF)
#define OPIsIRCAdmin(x) 	((x)->oflag & OFLAG_IRCADMIN)
#define OPIsServicesAdmin(x)	((x)->oflag & OFLAG_SERVICESADMIN)
#define OPIsNetAdmin(x)		((x)->oflag & OFLAG_NETADMIN)
#define OPIsServAdmin(x)	((x)->oflag & OFLAG_SERVADMIN)
#define OPIsTechAdmin(x)	((x)->oflag & OFLAG_TECHADMIN)
#define OPIsServicesRoot(x)	((x)->oflag & OFLAG_SERVICESROOT)
#define OPIsProtected(x)	((x)->oflag & OFLAG_PROTECTED)

#define OPSetRehash(x)		((x)->oflag |= OFLAG_REHASH)
#define OPSetDie(x)		((x)->oflag |= OFLAG_DIE)
#define OPSetRestart(x)		((x)->oflag |= OFLAG_RESTART)
#define OPSetHelpOp(x)		((x)->oflag |= OFLAG_HELPOP)
#define OPSetGlobOps(x)		((x)->oflag |= OFLAG_GLOBOP)
#define OPSetWallOps(x)		((x)->oflag |= OFLAG_WALLOP)
#define OPSetLocOps(x)		((x)->oflag |= OFLAG_LOCOP)
#define OPSetLRoute(x)		((x)->oflag |= OFLAG_LROUTE)
#define OPSetGRoute(x)		((x)->oflag |= OFLAG_GROUTE)
#define OPSetLKill(x)		((x)->oflag |= OFLAG_LKILL)
#define OPSetGKill(x)		((x)->oflag |= OFLAG_GKILL)
#define OPSetKline(x)		((x)->oflag |= OFLAG_KLINE)
#define OPSetUnKline(x)		((x)->oflag |= OFLAG_UNKLINE)
#define OPSetLNotice(x)		((x)->oflag |= OFLAG_LNOTICE)
#define OPSetGNotice(x)		((x)->oflag |= OFLAG_GNOTICE)
#define OPSSetAdmin(x)		((x)->oflag |= OFLAG_ADMIN)
#define OPSSetServicesOper(x) 	((x)->oflag |= OFLAG_SERVICESOPER)
#define OPSetUModeC(x)		((x)->oflag |= OFLAG_UMODEC)
#define OPSetUModeF(x)		((x)->oflag |= OFLAG_UMODEF)
#define OPSetZLine(x)	 	((x)->oflag |= OFLAG_ZLINE)
#define OPSetIRCAdmin(x) 	((x)->oflag |= OFLAG_IRCADMIN)
#define OPSetServicesAdmin(x)	((x)->oflag |= OFLAG_SERVICESADMIN)
#define OPSetNetAdmin(x)	((x)->oflag |= OFLAG_NETADMIN)
#define OPSetServAdmin(x)	((x)->oflag |= OFLAG_SERVADMIN)
#define OPSetTechAdmin(x)	((x)->oflag |= OFLAG_TECHADMIN)
#define OPSetServicesRoot(x)	((x)->oflag |= OFLAG_SERVICESROOT)
#define OPSetProtected(x)	((x)->oflag |= OFLAG_PROTECTED)

#define OPClearRehash(x)	((x)->oflag &= ~OFLAG_REHASH)
#define OPClearDie(x)		((x)->oflag &= ~OFLAG_DIE)  
#define OPClearRestart(x)	((x)->oflag &= ~OFLAG_RESTART)
#define OPClearHelpOp(x)	((x)->oflag &= ~OFLAG_HELPOP)
#define OPClearGlobOps(x)	((x)->oflag &= ~OFLAG_GLOBOP)
#define OPClearWallOps(x)	((x)->oflag &= ~OFLAG_WALLOP)
#define OPClearLocOps(x)	((x)->oflag &= ~OFLAG_LOCOP)
#define OPClearLRoute(x)	((x)->oflag &= ~OFLAG_LROUTE)
#define OPClearGRoute(x)	((x)->oflag &= ~OFLAG_GROUTE)
#define OPClearLKill(x)		((x)->oflag &= ~OFLAG_LKILL)
#define OPClearGKill(x)		((x)->oflag &= ~OFLAG_GKILL)
#define OPClearKline(x)		((x)->oflag &= ~OFLAG_KLINE)
#define OPClearUnKline(x)	((x)->oflag &= ~OFLAG_UNKLINE)
#define OPClearLNotice(x)	((x)->oflag &= ~OFLAG_LNOTICE)
#define OPClearGNotice(x)	((x)->oflag &= ~OFLAG_GNOTICE)
#define OPClearAdmin(x)		((x)->oflag &= ~OFLAG_ADMIN)
#define OPClearServicesOper(x)	((x)->oflag &= ~OFLAG_SERVICESOPER)
#define OPClearUModeC(x)	((x)->oflag &= ~OFLAG_UMODEC)
#define OPClearUModeF(x)	((x)->oflag &= ~OFLAG_UMODEF)
#define OPClearZLine(x)		((x)->oflag &= ~OFLAG_ZLINE)
#define OPClearIRCAdmin(x)	((x)->oflag &= ~OFLAG_IRCADMIN)
#define OPClearServicesAdmin(x)	((x)->oflag &= ~OFLAG_SERVICESADMIN)
#define OPClearNetAdmin(x)	((x)->oflag &= ~OFLAG_NETADMIN)
#define OPClearServAdmin(x)	((x)->oflag &= ~OFLAG_SERVADMIN)
#define OPClearTechAdmin(x)	((x)->oflag &= ~OFLAG_TECHADMIN)
#define OPClearServicesRoot(x)	((x)->oflag &= ~OFLAG_SERVICESROOT)
#define OPClearProtected(x)	((x)->oflag &= ~OFLAG_PROTECTED)


/*
 * defined debugging levels
 */
#define	DEBUG_FATAL  0
#define	DEBUG_ERROR  1	/* report_error() and other errors that are found */
#define	DEBUG_NOTICE 3
#define	DEBUG_DNS    4	/* used by all DNS related routines - a *lot* */
#define	DEBUG_INFO   5	/* general usful info */
#define	DEBUG_NUM    6	/* numerics */
#define	DEBUG_SEND   7	/* everything that is sent out */
#define	DEBUG_DEBUG  8	/* anything to do with debugging, ie unimportant :) */
#define	DEBUG_MALLOC 9	/* malloc/free calls */
#define	DEBUG_LIST  10	/* debug list use */


/*
 * UPING Start
 */

#define ClearPing(x)            ((x)->flags &= ~FLAGS_PING)
#define ClearAskedPing(x)       ((x)->flags &= ~FLAGS_ASKEDPING)
#define SetAskedPing(x)         ((x)->flags |= FLAGS_ASKEDPING)
#define AskedPing(x)            ((x)->flags & FLAGS_ASKEDPING)
#define DoPing(x)               ((x)->flags & FLAGS_PING)
#define IsPing(x)               ((x)->status == STAT_PING)
#define SetPing(x)              ((x)->status = STAT_PING)



/*
 * defines for curses in client
 */
#define	DUMMY_TERM	0
#define	CURSES_TERM	1
#define	TERMCAP_TERM	2

struct  SqlineItem	{
	unsigned int	status;
	char *sqline;
	char *reason;
	struct	SqlineItem *next;
};

struct	ConfItem	{
	unsigned int	status;	/* If CONF_ILLEGAL, delete when no clients */
	int	clients;	/* Number of *LOCAL* clients using this */
	struct	in_addr ipnum;	/* ip number of host field */
	char	*host;
	char	*passwd;
	char	*name;
	int	port;
	time_t	hold;	/* Hold action until this time (calendar time) */
	int	tmpconf;
#ifndef VMSP
	aClass	*class;  /* Class of connection */
#endif
	struct	ConfItem *next;
};

#define	CONF_ILLEGAL		0x80000000
#define	CONF_MATCH		0x40000000
#define	CONF_QUARANTINED_SERVER	0x0001
#define	CONF_CLIENT		0x0002
#define	CONF_CONNECT_SERVER	0x0004
#define	CONF_NOCONNECT_SERVER	0x0008
#define	CONF_LOCOP		0x0010
#define	CONF_OPERATOR		0x0020
#define	CONF_ME			0x0040
#define	CONF_KILL		0x0080
#define	CONF_ADMIN		0x0100

#define	CONF_CLASS		0x0400
#define	CONF_SERVICE		0x0800
#define	CONF_LEAF		0x1000
#define	CONF_LISTEN_PORT	0x2000
#define	CONF_HUB		0x4000
#define	CONF_UWORLD		0x8000
#define CONF_QUARANTINED_NICK	0x10000
#define CONF_ZAP		0x20000
#define CONF_CONFIG             0x100000
#define CONF_CRULEALL           0x200000
#define CONF_CRULEAUTO          0x400000
#define CONF_MISSING		0x800000
#define CONF_SERVICESOPER	0x1000000
#define CONF_DRPASS		0x2000000   /* DIE/RESTART pass - NikB */
#define CONF_EXCEPT		0x4000000   /* K-Line exeptions */
#define CONF_SOCKSEXCEPT	0x8000000   /* Socks Checking Exeptions */

#define	CONF_OPS		(CONF_OPERATOR | CONF_LOCOP)
#define	CONF_SERVER_MASK	(CONF_CONNECT_SERVER | CONF_NOCONNECT_SERVER)
#define	CONF_CLIENT_MASK	(CONF_CLIENT | CONF_SERVICE | CONF_OPS | \
				 CONF_SERVER_MASK)
#define CONF_CRULE              (CONF_CRULEALL | CONF_CRULEAUTO)
#define CONF_QUARANTINE		(CONF_QUARANTINED_SERVER|CONF_QUARANTINED_NICK)

#define	IsIllegal(x)	((x)->status & CONF_ILLEGAL)
#define IsTemp(x)	((x)->tmpconf)

/*
 * Client structures
 */
struct	User	{
	struct	User	*nextu;
	Link	*channel;	/* chain of channel pointer blocks */
	Link	*invited;	/* chain of invite pointer blocks */
	Link	*silence;	/* chain of silence pointer blocks */
	char	*away;		/* pointer to away message */
	time_t	last;
	u_int32_t servicestamp;  /* Services' time stamp variable */
	int	refcnt;		/* Number of times this block is referenced */
	int	joined;		/* number of channels joined */
	char	username[USERLEN+1];
	char	host[HOSTLEN+1];
	char	realhost[HOSTLEN+1];
	char	virthost[HOSTLEN+1];
        char	server[HOSTLEN+1];
        aClient *serv;

#ifdef	LIST_DEBUG
	aClient	*bcptr;
#endif
};

struct	Server	{
	struct	Server	*nexts;
	anUser	*user;		/* who activated this connection */
        aClient *uplink;	
	char	up[HOSTLEN+1];	/* uplink for this server */
	char	by[NICKLEN+1];
	aConfItem *nline;	/* N-line pointer for this server */
        time_t  timestamp;      /* Remotely determined connect try time */
        time_t  ghost;          /* Local time at which a new server caused a Ghost */
        u_short prot;           /* Major protocol */

#ifdef	LIST_DEBUG
	aClient	*bcptr;
#endif
};

/* struct s_gline {
	char   name[128];
	char   sname[128];
	char   host[128];
	char   reason[128];
	char   timemsg[128];
	time_t    bannedtime;
	time_t    bantime;
};*/

struct t_crline {
	char *channel;
	int  type;
	aCRline *next, *prev;
};

struct t_gline {
	char *usermask;
	char *hostmask;
	char *reason;
	char *setby;
	time_t expire_at;
	time_t set_at;
	aGline *next;
	aGline *prev;
};

struct Client	{
	struct	Client *next, *prev, *hnext;
	anUser	*user;		/* ...defined, if this is a User */
	aServer	*serv;		/* ...defined, if this is a server */
        int     tag;            /* tag */
	int     iown;           /* 1 if mine, 0 if not */
	int	hashv;		/* raw hash value (according to ircu) -- was
				 * added so that UPING would work. Dreamforge
				 * 4.6.7b Not needed in b ? I dunno they said so.
				 */
	time_t	lasttime;	/* ...should be only LOCAL clients? --msa */
	time_t	firsttime;	/* time client was created */
	time_t	since;		/* last time we parsed something */
	time_t	lastnick;	/* TimeStamp on nick */
	time_t  nextnick;	/* Time the next nick change will be allowed */
	time_t  nexttarget;	/* Time until a change in targets is allowed */
	u_char targets[MAXTARGETS]; /* Hash values of current targets */	
	unsigned long	flags;	/* client flags */
	unsigned long	umodes;	/* client usermodes */
	unsigned long	umodes2; /* client usermodes 2 */
	aClient	*from;		/* == self, if Local Client, *NEVER* NULL! */
	int	fd;		/* >= 0, for local clients */
	int	hopcount;	/* number of servers to this 0 = local */
        int     curruser;       /* Number of users on this server */
	short	status;		/* Client type */
	char	name[HOSTLEN+1]; /* Unique name of the client, nick or host */
	char	username[USERLEN+1]; /* username here now for auth stuff */
	char	info[REALLEN+1]; /* Free form additional client information */
	aClient	*srvptr;	/* Server introducing this.  May be &me */
	Link	*history;	/* Whowas linked list */
	/*
	** The following fields are allocated only for local clients
	** (directly connected to *this* server with a socket.
	** The first of them *MUST* be the "count"--it is the field
	** to which the allocation is tied to! *Never* refer to
	** these fields, if (from != self).
	*/
	int	count;		/* Amount of data in buffer */
	char	buffer[BUFSIZE]; /* Incoming message buffer */
	short	lastsq;		/* # of 2k blocks when sendqueued called last*/
	dbuf	sendQ;		/* Outgoing message queue--if socket full */
	dbuf	recvQ;		/* Hold for data incoming yet to be parsed */
#ifdef NOSPOOF
	u_int32_t	nospoof;	/* Anti-spoofing random number */
#endif
	long	oflag;		/* Operator access flags -Cabal95 */
	long	proto;		/* ProtoCtl options */
	long	sendM;		/* Statistics: protocol messages send */
	long	sendK;		/* Statistics: total k-bytes send */
	long	receiveM;	/* Statistics: protocol messages received */
	long	receiveK;	/* Statistics: total k-bytes received */
	u_short	sendB;		/* counters to count upto 1-k lots of bytes */
	u_short	receiveB;	/* sent and received. */
	aClient	*acpt;		/* listening client which we accepted from */
	Link	*confs;		/* Configuration record associated */
	int	authfd;		/* fd for rfc931 authentication */
	int	socksfd;	/* fd for rfc1928 checking */
	struct	in_addr	ip;	/* keep real ip# too */
        char        hostip[15 + 1];  /* Aswell as the real IP as a string */
	u_short	port;		/* and the remote port# too :-) */
	struct	hostent	*hostp;
	u_short	notifies;	/* Keep track of count of notifies */
	Link	*notify;	/* Links to clients notify-structures */
	LOpts	*lopt;		/* Saved /list options */
#ifdef	pyr
	struct	timeval	lw;
#endif
	char	sockhost[HOSTLEN+1]; /* This is the host name from the socket
				     ** and after which the connection was
				     ** accepted.
				     */
	char	passwd[PASSWDLEN+1];
#ifdef DEBUGMODE
	time_t	cputime;
#endif
};

#define	CLIENT_LOCAL_SIZE sizeof(aClient)
#define	CLIENT_REMOTE_SIZE offsetof(aClient,count)

/*
 * statistics structures
 */
struct	stats {
	unsigned int	is_cl;	/* number of client connections */
	unsigned int	is_sv;	/* number of server connections */
	unsigned int	is_ni;	/* connection but no idea who it was */
	unsigned short	is_cbs;	/* bytes sent to clients */
	unsigned short	is_cbr;	/* bytes received to clients */
	unsigned short	is_sbs;	/* bytes sent to servers */
	unsigned short	is_sbr;	/* bytes received to servers */
	unsigned long	is_cks;	/* k-bytes sent to clients */
	unsigned long	is_ckr;	/* k-bytes received to clients */
	unsigned long	is_sks;	/* k-bytes sent to servers */
	unsigned long	is_skr;	/* k-bytes received to servers */
	time_t 		is_cti;	/* time spent connected by clients */
	time_t		is_sti;	/* time spent connected by servers */
	unsigned int	is_ac;	/* connections accepted */
	unsigned int	is_ref;	/* accepts refused */
	unsigned int	is_unco; /* unknown commands */
	unsigned int	is_wrdi; /* command going in wrong direction */
	unsigned int	is_unpf; /* unknown prefix */
	unsigned int	is_empt; /* empty message */
	unsigned int	is_num;	/* numeric message */
	unsigned int	is_kill; /* number of kills generated on collisions */
	unsigned int	is_fake; /* MODE 'fakes' */
	unsigned int	is_asuc; /* successful auth requests */
	unsigned int	is_abad; /* bad auth requests */
	unsigned int	is_udp;	/* packets recv'd on udp port */
	unsigned int	is_loc;	/* local connections made */
};

struct ListOptions {
	LOpts	*next;
	Link	*yeslist, *nolist;
	int	starthash;
	short int	showall;
	unsigned short	usermin;
	int	usermax;
	time_t	currenttime;
	time_t	chantimemin;
	time_t	chantimemax;
	time_t	topictimemin;
	time_t	topictimemax;
};

/* mode structure for channels */

struct	SMode	{
#ifndef USE_LONGMODE
	unsigned int 	mode;
#else
	long	mode;
#endif
	int	limit;
	char	key[KEYLEN+1];
	char	link[LINKLEN+1];
        /* x:y */
	int	msgs; /* x */
	int	per;  /* y */
	int	kmode; /* mode  0 = kick  1 = ban */
};

/* Message table structure */

struct	Message	{
	char	*cmd;
	int	(* func)();
	unsigned int	count;
	int	parameters;
	char	flags;
		/* bit 0 set means that this command is allowed to be used
		 * only on the average of once per 2 seconds -SRB */
	u_char	token[2]; /* Cheat for tokenized value */
	unsigned long	bytes;
#ifdef DEBUGMODE
	unsigned long	lticks;
	unsigned long	rticks;
#endif
};

/* Used for notify-hash buckets... -Donwulff */

struct Notify {
	aNotify	*hnext;
	time_t	lasttime;
	Link	*notify;
	char	nick[1];
};

/* general link structure used for chains */

struct	SLink	{
	struct	SLink	*next;
	int	  flags;
	aFloodOpt *flood;
	union {
		aClient	*cptr;
		aChannel *chptr;
		aConfItem *aconf;
		aNotify	*nptr;
		aName *whowas;
		char	*cp;
                struct {
                  char *banstr;
                  char *who;
                  time_t when;
                } ban;
	} value;
};

struct	SBan	{
	struct	SBan	*next;
	char *banstr;
	char *who;
	time_t when;
};

struct DSlink {
        struct  DSlink  *next;
        struct  DSlink  *prev;
        union {
                aClient *cptr;
                aChannel *chptr;
                aConfItem *aconf;
                char    *cp;
        } value;
};

/* channel structure */

struct Channel	{
	struct	Channel *nextch, *prevch, *hnextch;
	Mode	mode;
	time_t	creationtime;
	char	topic[TOPICLEN+1];
	char	topic_nick[NICKLEN+1];
	time_t	topic_time;
	int	users;
	Link	*members;
	Link	*invites;
	Link	*blist;
	Ban	*banlist;
	Ban 	*exlist; /* exceptions */
	char	chname[1];
};

/*
** Channel Related macros follow
*/

/* Channel related flags */

#define	CHFL_CHANOP     0x0001 /* Channel operator */
#define	CHFL_VOICE      0x0002 /* the power to speak */
#define CHFL_HALFOP	0x0004 /* Channel Half Operator */
#define	CHFL_DEOPPED	0x0008 /* Is de-opped by a server */
#define	CHFL_SERVOPOK   0x0010 /* Server op allowed */
#define	CHFL_ZOMBIE     0x0020 /* Kicked from channel */
/* Bans are stored in separate linked list, so phase this out? */
#define	CHFL_BAN	0x0040 /* ban channel flag */
#define CHFL_EXCEPT	0x0080 /* ban excemption flag */
#define	CHFL_OVERLAP    (CHFL_CHANOP|CHFL_HALFOP|CHFL_VOICE)

/* Channel Visibility macros */

#define	MODE_CHANOP	CHFL_CHANOP
#define MODE_HALFOP	CHFL_HALFOP
#define	MODE_VOICE	CHFL_VOICE
#define	MODE_PRIVATE	0x0008
#define	MODE_SECRET	0x0010
#define	MODE_MODERATED  0x0020
#define	MODE_TOPICLIMIT 0x0040
#define	MODE_INVITEONLY 0x0080
#define	MODE_NOPRIVMSGS 0x0100
#define	MODE_KEY	0x0200
#define MODE_EXCEPT	0x0400
#define	MODE_BAN	0x0800
#define	MODE_LIMIT	0x1000
#define MODE_RGSTR	0x2000
#define MODE_RGSTRONLY  0x4000

#define MODE_OPERONLY   0x8000
#define MODE_ADMONLY   	0x10000
#define MODE_LINK	0x20000
#define MODE_NOCOLOR	0x40000
#define MODE_STRIP	0x80000
#define MODE_NOKNOCK	0x100000
#define MODE_NOINVITE  	0x200000
#define MODE_FLOODLIMIT 0x400000

#define is_halfop is_half_op

/*
 * mode flags which take another parameter (With PARAmeterS)
 */
#define	MODE_WPARAS	(MODE_CHANOP|MODE_HALFOP|MODE_VOICE|MODE_BAN|MODE_KEY|MODE_LIMIT|MODE_LINKMODE_EXCEPT)
/*
 * Undefined here, these are used in conjunction with the above modes in
 * the source.
#define	MODE_DEL       0x200000000
#define	MODE_ADD       0x400000000
 */

#define	HoldChannel(x)		(!(x))
/* name invisible */
#define	SecretChannel(x)	((x) && ((x)->mode.mode & MODE_SECRET))
/* channel not shown but names are */
#define	HiddenChannel(x)	((x) && ((x)->mode.mode & MODE_PRIVATE))
/* channel visible */
#define	ShowChannel(v,c)	(PubChannel(c) || IsMember((v),(c)))
#define	PubChannel(x)		((!x) || ((x)->mode.mode &\
				 (MODE_PRIVATE | MODE_SECRET)) == 0)

#define	IsChannelName(name) ((name) && (*(name) == '#' || *(name) == '&' || *(name) == '+'))
#define IsModelessChannel(name) ((name) && (*(name) == '+'))


struct s_gline {
	char   name[128];
	char   sname[128];
	char   host[128];
	char   reason[128];
	char   timemsg[128];
	long   expire_at;
	long   set_at;
};


struct FloodOpt {
	int	nmsg;
	time_t  lastmsg;
 };

/* Misc macros */

#define	BadPtr(x) (!(x) || (*(x) == '\0'))

#define	isvalid(c) (((c) >= 'A' && (c) <= '~') || isdigit(c) || (c) == '-')

#define	MyConnect(x)			((x)->fd >= 0)
#define	MyClient(x)			(MyConnect(x) && IsClient(x))
#define	MyOper(x)			(MyConnect(x) && IsOper(x))

#define TStime() (time(NULL)+TSoffset)

/* Lifted somewhat from Undernet code --Rak */

#define IsSendable(x)		(DBufLength(&x->sendQ) < 2048)
#define DoList(x)		((x)->lopt)

/* String manipulation macros */

/* strncopynt --> strncpyzt to avoid confusion, sematics changed
   N must be now the number of bytes in the array --msa */
#define	strncpyzt(x, y, N) do{(void)strncpy(x,y,N);x[N-1]='\0';}while(0)
#define	StrEq(x,y)	(!strcmp((x),(y)))

/* used in SetMode() in channel.c and m_umode() in s_msg.c */

#define	MODE_NULL      0
#define	MODE_ADD       0x40000000
#define	MODE_DEL       0x20000000

/* return values for hunt_server() */

#define	HUNTED_NOSUCH	(-1)	/* if the hunted server is not found */
#define	HUNTED_ISME	0	/* if this server should execute the command */
#define	HUNTED_PASS	1	/* if message passed onwards successfully */

/* used when sending to #mask or $mask */

#define	MATCH_SERVER  1
#define	MATCH_HOST    2

/* used for async dns values */

#define	ASYNC_NONE	(-1)
#define	ASYNC_CLIENT	0
#define	ASYNC_CONNECT	1
#define	ASYNC_CONF	2
#define	ASYNC_SERVER	3
#define ASYNC_PING	4	/* for UPING */

/* misc variable externs */

extern	char	*credittext[];
extern	char	*generation, *creation, *version;

/* misc defines */

#define	FLUSH_BUFFER	-2
#define	UTMP		"/etc/utmp"
#define	COMMA		","

#define     UDPPORT         "7007"	/* For UPING */


#endif /* __struct_include__ */
